@@ -22,7 +22,6 @@
 */
 
 #include <config.h>
-
 #include "rsvg.h"
 
 #include <string.h>
@@ -118,6 +117,8 @@ struct _RsvgSaxHandler {
   void (*characters) (RsvgSaxHandler *self, const xmlChar *ch, int len);
 };
 
+char *fonts_dir;
+
 static RsvgCtx *
 rsvg_ctx_new (void)
 {
@@ -923,6 +924,8 @@ rsvg_text_handler_characters (RsvgSaxHandler *self, const xmlChar *ch, int len)
   GdkPixbuf *pixbuf;
   gboolean has_alpha;
   int opacity;
+  const char *dir;
+  char *path;
 
   /* Copy ch into string, chopping off leading and trailing whitespace */
   for (beg = 0; beg < len; beg++)
@@ -937,7 +940,6 @@ rsvg_text_handler_characters (RsvgSaxHandler *self, const xmlChar *ch, int len)
   memcpy (string, ch + beg, end - beg);
   string[end - beg] = 0;
 
-
 #ifdef VERBOSE
   fprintf (stderr, "text characters(%s, %d)\n", string, len);
 #endif
@@ -945,14 +947,21 @@ rsvg_text_handler_characters (RsvgSaxHandler *self, const xmlChar *ch, int len)
   if (ctx->ft_ctx == NULL)
     ctx->ft_ctx = rsvg_ft_ctx_new ();
 
-  /* FIXME bugzilla.eazel.com 3904: We need to make rsvg use the 
-   * Nautilus font mapping stuff in NautilusScalableFont.  See bug
-   * for details.
+  /* FIXME bugzilla.eazel.com 3904: We need to make rsvg use something
+   * like the Nautilus font mapping stuff in NautilusScalableFont. See
+   * bug for details.
    */
-  fh = rsvg_ft_intern (ctx->ft_ctx,
-		       NAUTILUS_DATADIR "/fonts/urw/n019003l.pfb");
-  rsvg_ft_font_attach (ctx->ft_ctx, fh,
-		       NAUTILUS_DATADIR "/fonts/urw/n019003l.afm");
+  if (fonts_dir == NULL) {
+    dir = DATADIR "/eel/fonts";
+  } else {
+    dir = fonts_dir;
+  }
+  path = g_strconcat (dir, "/urw/n019003l.pfb", NULL);
+  fh = rsvg_ft_intern (ctx->ft_ctx, path);
+  g_free (path);
+  path = g_strconcat (dir, "/urw/n019003l.afm", NULL);
+  rsvg_ft_font_attach (ctx->ft_ctx, fh, path);
+  g_free (path);
 
   state = &ctx->state[ctx->n_state - 1];
 
@@ -978,19 +987,25 @@ rsvg_text_handler_characters (RsvgSaxHandler *self, const xmlChar *ch, int len)
 				     state->font_size, state->font_size,
 				     state->affine, glyph_xy);
 
-      rsvg_render_paint_server (render, state->fill, NULL); /* todo: paint server ctx */
-      opacity = state->fill_opacity * state->opacity;
-      opacity = opacity + (opacity >> 7) + (opacity >> 14);
+      if (glyph == NULL)
+	{
+	}
+      else
+	{
+	  rsvg_render_paint_server (render, state->fill, NULL); /* todo: paint server ctx */
+	  opacity = state->fill_opacity * state->opacity;
+	  opacity = opacity + (opacity >> 7) + (opacity >> 14);
 #ifdef VERBOSE
-      fprintf (stderr, "opacity = %d\n", opacity);
+	  fprintf (stderr, "opacity = %d\n", opacity);
 #endif
-      art_render_mask_solid (render, opacity);
-      art_render_mask (render,
-		       glyph_xy[0], glyph_xy[1],
-		       glyph_xy[0] + glyph->width, glyph_xy[1] + glyph->height,
-		       glyph->buf, glyph->rowstride);
-      art_render_invoke (render);
-      rsvg_ft_glyph_unref (glyph);
+	  art_render_mask_solid (render, opacity);
+	  art_render_mask (render,
+			   glyph_xy[0], glyph_xy[1],
+			   glyph_xy[0] + glyph->width, glyph_xy[1] + glyph->height,
+			   glyph->buf, glyph->rowstride);
+	  art_render_invoke (render);
+	  rsvg_ft_glyph_unref (glyph);
+	}
     }
 
   g_free (string);
@@ -1392,6 +1407,13 @@ static xmlSAXHandler rsvgSAXHandlerStruct = {
 
 static xmlSAXHandlerPtr rsvgSAXHandler = &rsvgSAXHandlerStruct;
 
+void
+rsvg_set_fonts_dir (const char *new_fonts_dir)
+{
+  g_free (fonts_dir);
+  fonts_dir = g_strdup (new_fonts_dir);
+}
+
 GdkPixbuf *
 rsvg_render_file (FILE *f, double zoom)
 {
